/**
 * @file
 * 
 * @brief Removed legacy symbols for an outdated FTP client
 * 
 *
 * @copyright See Copyright for the status of this software.
 *
 * @author Daniel Veillard
 */

#ifndef __NANO_FTP_H__
#define __NANO_FTP_H__

#ifdef __GNUC__
  #warning "libxml/nanoftp.h is deprecated"
#endif

#endif /* __NANO_FTP_H__ */
